package de.ba_leipzig.discoinv.db;

import java.io.File;
import java.sql.SQLException;
import java.util.*;

public interface DatabaseManager {
	public Set<Beverage> getBeverages() throws SQLException;
	public int getDBManagerID();
	public int getDBUpdateCount();
	public Set<InvObject> getMisc() throws SQLException;
	public Set<Place> getPlaces() throws SQLException;
	public Set<PlaceWithObjects> getPlacesWithObjects() throws SQLException;
	public Set<? extends AssociationInfo<? extends InvObject>>
		getAssociatedObjects(Place place) throws SQLException;
	public Set<? extends AssociationInfo<? extends Place>>
		getAssociatedPlaces(InvObject object) throws SQLException;
	public PlaceWithObjects getPlaceWithObjects(int id) throws SQLException;
	public InvObject getInvObject(int id) throws SQLException;
	public int getTotalPresent(InvObject object) throws SQLException;
	public int getTotalNeeded(InvObject object) throws SQLException;
	public boolean isOpenedDB();
	public void closeDB() throws SQLException;
	public void openDB(File file) throws SQLException, DBIntegrityException;
	public void saveDBAs(File file);
	public void addPlace(Place place)
		throws SQLException, DBIntegrityException;
	public void addInvObject(InvObject object)
		throws SQLException, DBIntegrityException;
	public Set<BeverageType> getBeverageTypes() throws SQLException;
	public void addBeverage(Beverage beverage)
		throws SQLException, DBIntegrityException;
	public void addBeverage(BeverageWithLevel beverage)
		throws SQLException, DBIntegrityException;
	public void newDB(File file) throws SQLException;
	public void renamePlace(int id, String newName) throws SQLException;
	public void renameInvObject(int id, String newName) throws SQLException;
	public void deletePlace(int id) throws SQLException, DBIntegrityException;
	public void deleteInvObject(int id)
		throws SQLException, DBIntegrityException;
	public void updatePlace(Place place) throws SQLException;
	public void updateBeverage(Beverage beverage) throws SQLException;
	public void updateBeverage(BeverageWithLevel beverage) throws SQLException;
	
	/**
	 * Retrieves all beverages (with level) which are currently not associated
	 * to the given place.
	 * 
	 * @param	place The place for which the beverages (with level) which are
	 * 			not associated with it are to be retrieved
	 * 
	 * @return	The beverages (with level) which are currently not associated
	 * 			to the given place
	 * 
	 * @throws	SQLException in case of an database error
	 */
	public Set<Beverage> getBeveragesNotAtPlace(Place place)
		throws SQLException;
	
	/**
	 * Retrieves all misc. objects which are currently not associated with the
	 * given place.
	 * 
	 * @param	place The place for which the misc. objects which are not
	 * 			associated with it are to be retrieved
	 * 
	 * @return	The misc. objects which are currently not associated with the
	 * 			given place
	 * 
	 * @throws	SQLException in case of an database error
	 */
	public Set<InvObject> getMiscNotAtPlace(Place place) throws SQLException;
	
	/**
	 * Retrieves all places the given object is currently not associated with.
	 * The object may either be a beverage (with level) or a misc. object.
	 * 
	 * @param	object The object for which the places it is not associated
	 * 			with are to be retrieved
	 * 
	 * @return	The places the given object is currently not associated with.
	 * 
	 * @throws	SQLException in case of an database error
	 */
	public Set<Place> getPlacesObjectNotAt(InvObject object)
		throws SQLException;
	
	/**
	 * Determines whether there exists at least one place in the database.
	 *  
	 * @return	true if and only if there exists at least one place in the
	 * 			database, false otherwise
	 *  
	 * @throws 	SQLException in case of an database error
	 */
	public boolean existsPlace() throws SQLException;
	
	/**
	 * Determines whether there exists at least one beverage (with level) in
	 * the database.
	 * 
	 * @return	true if and only if there exists at least one beverage in the
	 * 			database, false otherwise
	 * 
	 * @throws	SQLException in case of an database error
	 */
	public boolean existsBeverage() throws SQLException;
	
	/**
	 * Determines whether there exists at least one misc. in the database.
	 * 
	 * @return	true if and only if there exists at least one misc. in the
	 * 			database, false otherwise
	 * 
	 * @throws	SQLException in case of an database error
	 */
	public boolean existsMisc() throws SQLException;
	
	/**
	 * Determines whether there are any beverages (with level) associated with
	 * the given place.
	 *  
	 * @param	place The place for which the beverage associations are to
	 * 			be checked
	 * 
	 * @return	true if and only if there is at least one beverage associated
	 * 			to the given place, false otherwise
	 * 
	 * @throws	SQLException in case of an database error
	 */
	public boolean hasBeverageAssociations(Place place) throws SQLException;
	
	/**
	 * Determines whether there are any misc. objects associated with the given
	 * place.
	 *  
	 * @param	place The place for which the misc. associations are to
	 * 			be checked
	 * 
	 * @return	true if and only if there is at least one misc. object
	 * 			associated with the given place, false otherwise
	 * 
	 * @throws	SQLException in case of an database error
	 */
	public boolean hasMiscAssociations(Place place) throws SQLException;
	
	/**
	 * Determines whether there are any places associated with the given object.
	 * The object can either be a beverage (with level) or a misc. object.
	 * 
	 * @param	object The object for which the place associations are to be
	 * 			checked
	 * 
	 * @return	true if and only if there is at least one place
	 * 			associated with the given object, false otherwise
	 * 
	 * @throws	SQLException in case of an database error
	 */
	public boolean hasPlaceAssociations(InvObject object) throws SQLException;
	
	/**
	 * Determines whether there exists at least one beverage (with level) that's
	 * currently not associated with the place given.
	 * 
	 * @param	place The place for which the check is to be conducted
	 * 
	 * @return	true if and only if there exists at least one beverage that's
	 * 			currently not associated with the given place, false otherwise
	 * 
	 * @throws	SQLException in case of an database error
	 */
	public boolean existsBeverageNotAtPlace(Place place) throws SQLException;
	
	/**
	 * Determines whether there exists at least one misc. object that's
	 * currently not associated with the place given.
	 * 
	 * @param	place The place for which the check is to be conducted
	 * 
	 * @return	true if and only if there exists at least one misc. object
	 * 			that's currently not associated with the given place, false
	 * 			otherwise
	 * 
	 * @throws	SQLException in case of an database error
	 */
	public boolean existsMiscNotAtPlace(Place place) throws SQLException;
	
	/**
	 * Determines whether there exists at least one place the given object
	 * isn't associated with. The object may either be a beverage (with level)
	 * or a misc. object.
	 * 
	 * @param	object The object for which the check is to be conducted
	 * 
	 * @return	true if and only if there exists at least one place	the given
	 * 			object is currently not associated with, false otherwise
	 * 
	 * @throws	SQLException in case of an database error
	 */
	public boolean existsPlaceObjectNotAt(InvObject object) throws SQLException;
	
	/**
	 * Adds new object associations to the given place.
	 * 
	 * <p>
	 * <b>Note:</b>
	 * <p>
	 * The object that can be retrieved by calling
	 * {@link AssociationInfo#getObject()} for any element in the associations
	 * set may either be a beverage (with level) or a misc. object. The set
	 * must always contain at least one element.
	 * 
	 * @param 	place The place to which the object associations are to be
	 * 			added
	 * 
	 * @param	associations The associations to be added to the given place
	 * 
	 * @throws	SQLException in case of an database error
	 * 
	 * @throws	DBIntegrityError in case a call to
	 * 			{@link AssociationInfo#getObject()} yields a place instead of
	 * 			a beverage (with level) or a misc. object
	 * 
	 * @see		AssociationInfo
	 */
	public void addObjectAssociations(Place place,
			Set<? extends AssociationInfo<? extends Identifiable>> associations)
				throws SQLException, DBIntegrityError;
	
	/**
	 * Adds new place associations to the given object, which may either be a
	 * beverage (with level) or a misc. object.
	 * 
	 * <p>
	 * <b>Note:</b>
	 * <p>
	 * The object that can be retrieved by calling
	 * {@link AssociationInfo#getObject()} for any element in the associations
	 * set must be a place. The set	must always contain	at least one element.
	 * 
	 * @param	object The object to which the place associations are to be
	 * 			added
	 * 
	 * @param	associations The associations to be added to the given object
	 * 
	 * @throws	SQLException in case of an database error
	 * 
	 * @throws	DBIntegrityError in case a call to
	 * 			{@link AssociationInfo#getObject()} yields a beverage (with
	 * 			level) or a misc. object instead of a place
	 *
	 * @see		AssociationInfo
	 */
	public void addPlaceAssociations(InvObject object,
			Set<? extends AssociationInfo<? extends Identifiable>> associations)
				throws SQLException, DBIntegrityError;
	
	/**
	 * Updates the object associations for a given place.
	 * 
	 * <p>
	 * <b>Note:</b>
	 * <p>
	 * The object that can be retrieved by calling
	 * {@link AssociationInfo#getObject()} for any element in the associations
	 * set may either be a beverage	(with level) or a misc. object.
	 * <p>
	 * The associations set must not contain any elements for which the call to
	 * {@link AssociationInfo#getObject()} results in an object that's
	 * currently not associated with the given place. It may however <b>not</b>
	 * contain {@link AssociationInfo} objects for any or all of the beverages
	 * (with level) or misc. objects which in fact are associated with the place
	 * given. The associations for those beverages (with level) or misc.
	 * objects not contained in the set will be <b>deleted</b>.
	 * <p>
	 * Put another way this means that calling this method yields the same
	 * result as deleting all object associations for the given place and
	 * calling {@link DatabaseManager#addObjectAssociations(Place, Set)}
	 * for any associations set.
	 * 
	 * @param	place The place for which the object associations are to be
	 * 			updated
	 * 
	 * @param	associations The associations to be updated
	 * 
	 * @throws	SQLException in case of an database error
	 *
	 * @throws	DBIntegrityError in case a call to
	 * 			{@link AssociationInfo#getObject()} yields a place instead of
	 * 			a beverage (with level) or a misc. object
	 * 
	 * @see		AssociationInfo
	 */
	public void updateObjectAssociations(Place place,
			Set<? extends AssociationInfo<? extends Identifiable>> associations)
				throws SQLException, DBIntegrityError;
	
	/**
	 * Updates the place associations for a given object, which may be a
	 * beverage (with level) or a misc. object.
	 * 
	 * <p>
	 * <b>Note:</b>
	 * <p>
	 * The object that can be retrieved by calling
	 * {@link AssociationInfo#getObject()} for any element in the associations
	 * set must be a place.
	 * <p>
	 * The associations set must not contain any elements for which the call to
	 * {@link AssociationInfo#getObject()} results in a place the given object
	 * is not currently associated with. It may however <b>not</b> contain
	 * {@link AssociationInfo} objects for any or all of the places to which
	 * the given object is in fact associated. The associations for those
	 * places not contained in the set will be <b>deleted</b>.
	 * <p>
	 * Put another way this means that calling this method yields the same
	 * result as deleting all place associations for the given object and
	 * calling {@link DatabaseManager#addPlaceAssociations(InvObject, Set)}
	 * for any associations set.
	 * 
	 * @param	object The object for which the place associations are to be
	 * 			updated
	 * 
	 * @param	associations The associations to be updated
	 * 
	 * @throws	SQLException in case of an database error
	 *
	 * @throws	DBIntegrityError in case a call to
	 * 			{@link AssociationInfo#getObject()} yields a beverage (with
	 * 			level) or a misc. object instead of a place
	 *
	 * @see		AssociationInfo
	 */
	public void updatePlaceAssociations(InvObject object,
			Set<? extends AssociationInfo<? extends Identifiable>> associations)
				throws SQLException;
}