package de.ba_leipzig.discoinv.tree;

import java.util.*;

import javax.swing.tree.*;

import de.matthiasrothe.util.collections.*;

public class TreeModel extends DefaultTreeModel {
	private class TreeNodeAccessor implements DataAccessorStrategy<String> {
		private TreeNode parent;
		
		public TreeNodeAccessor(TreeNode parent){
			if(parent == null){
				throw new IllegalArgumentException("parent must not be null!");
			}
			this.parent = parent;
		}
		
		public String getElementAt(int index) {
			return ((TreeNode)parent.getChildAt(index)).getUserObject().toString();
		}

		public int getSize() {
			return parent.getChildCount();
		}
	}
	
	private static final long serialVersionUID = 1L;
	
	private Vector<TreePathValueChangeListener> valueChangeListeners =
		new Vector<TreePathValueChangeListener>();
	
	private TreeNode root =
		new TreeNode(TreeObjectType.baseTypes.get(TreeObjectType.root));
	private TreeNode places =
		new TreeNode(TreeObjectType.baseTypes.get(TreeObjectType.place));
	private TreeNode beverages =
		new TreeNode(TreeObjectType.baseTypes.get(TreeObjectType.beverage));
	private TreeNode misc =
		new TreeNode(TreeObjectType.baseTypes.get(TreeObjectType.misc));
	
	public TreeModel(){
		super(null, true);
		initRoot();
		setRoot(root);
	}
	
	private void initRoot(){
		insertNodeInto(places, root, 0);
		insertNodeInto(beverages, root, 1);
		insertNodeInto(misc, root, 2);
	}
	
	public void addChildToPlaces(TreeNode child){
		int position = getNodePosition(child, places);
		insertNodeInto(child, places, position);
	}
	
	public void addChildToBeverages(TreeNode child){
		int position = getNodePosition(child, beverages);
		insertNodeInto(child, beverages, position);
	}
	
	public void addChildToMisc(TreeNode child){
		int position = getNodePosition(child, misc);
		insertNodeInto(child, misc, position);
	}
	
	public void addTreePathValueChangeListener(TreePathValueChangeListener listener){
		if(listener == null){
			throw new IllegalArgumentException("listener must not be null!");
		}
		
		valueChangeListeners.add(listener);
	}
	
	public TreeNode getRoot(){
		return root;
	}
	
	public void removeTreePathValueChangeListener(TreePathValueChangeListener listener){
		valueChangeListeners.remove(listener);
	}
	
	public void unloadBeverages(){
		removeAllChildren(beverages);
	}
	
	public void unloadMisc(){
		removeAllChildren(misc);
	}
	
	public void unloadPlaces(){
		removeAllChildren(places);
	}
	
	public void valueForPathChanged(TreePath path, Object newValue){
		fireValueForPathChanged(path, newValue.toString());
		nodeChanged((TreeNode)path.getLastPathComponent());
	}
	
	private void fireValueForPathChanged(TreePath path, String newValue){
		for(TreePathValueChangeListener listener : valueChangeListeners){
			listener.valueForPathChanged(path, newValue);
		}
	}
	
	private int getNodePosition(TreeNode node, TreeNode parent){
		TreeNodeAccessor accessor = new TreeNodeAccessor(parent);
		
		OrderedIndexFinder<String> finder =
			new OrderedIndexFinder<String>(accessor);
		
		String input = node.getUserObject().toString();
		
		return finder.getOrderedIndex(input);
	}
	
	private void removeAllChildren(TreeNode parent){
		Vector<TreeNode> children = new Vector<TreeNode>();
		
		for(Enumeration childrenEnum = parent.children();
			childrenEnum.hasMoreElements();)
		{
			children.add((TreeNode)childrenEnum.nextElement());
		}
		
		for(TreeNode child : children){
			removeNodeFromParent(child);
		}
	}
}