package de.matthiasrothe.util.measuring;

import java.text.*;
import java.util.*;

public abstract class AbstractMeasurable<V extends Number, U extends MeasuringUnit>
implements Measurable<V, U> {
	private V value;
	private U unit;
	
	public AbstractMeasurable(U unit){
		if(unit == null){
			throw new IllegalArgumentException("unit must not be null!");
		}
		
		this.unit = unit;
	}
	
	public AbstractMeasurable(V value, U unit){
		if(value == null){
			throw new IllegalArgumentException("value must not be null!");
		}
		if(unit == null){
			throw new IllegalArgumentException("unit must not be null!");
		}
		
		this.value = value;
		this.unit = unit;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public U getUnit() {
		return unit;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public V getValue() {
		return value;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String getValueAndUnit(RetrievalMode mode) {
		String valueAndUnit = "";
		
		switch(mode){
			case abbreviated:
				valueAndUnit = getAbbreviation();
				break;
			case fullname:
				valueAndUnit = getFullname();
				break;
		}
		
		return valueAndUnit;
	}
	
	private String getAbbreviation(){
		return getValueString() + " " + unit.getAbbreviation();
	}
	
	private String getFullname(){
		return getValueString() + " " + unit.getFullName();
	}
	
	private String getValueString(){
		return NumberFormat.getInstance().format(value);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String getValueAndUnit(RetrievalMode mode, Locale locale){
		String valueAndUnit = "";
		
		switch(mode){
			case abbreviated:
				valueAndUnit = getAbbreviation(locale);
				break;
			case fullname:
				valueAndUnit = getFullname(locale);
				break;
		}
		
		return valueAndUnit;
	}
	
	private String getAbbreviation(Locale locale){
		return getValueString(locale) + " " + unit.getAbbreviation(locale);
	}
	
	private String getFullname(Locale locale){
		return getValueString(locale) + " " + unit.getFullName(locale);
	}
	
	private String getValueString(Locale locale){
		return NumberFormat.getInstance(locale).format(value);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void setValue(V value){
		if(value == null){
			throw new IllegalArgumentException("value must not be null!");
		}
		
		this.value = value;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public int compareTo(Measurable<V, U> o){
		Double thisValue =
			value.doubleValue() * unit.getPrefix().getMultiplicator();
		
		Double otherValue =
			o.getValue().doubleValue() *
			o.getUnit().getPrefix().getMultiplicator();
		
		return thisValue.compareTo(otherValue);
	}
	
	public String toString(){
		return getAbbreviation();
	}
}