package de.matthiasrothe.util.measuring;

import java.util.*;

public abstract class AbstractMeasuringUnit implements MeasuringUnit {
	private Prefix prefix;
	
	/**
	 * Constructs a new object and sets the prefix to {@link Prefix#none}.
	 */
	public AbstractMeasuringUnit(){
		this.prefix = Prefix.none;
	}
	
	/**
	 * Constructs a new object and sets the prefix to the prefix provided.
	 * 
	 * @param prefix The {@link Prefix} to be used with the new instance
	 */
	public AbstractMeasuringUnit(Prefix prefix){
		setPrefix(prefix);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public int compareTo(MeasuringUnit unit){
		if(getClass().equals(unit.getClass())){
			return getPrefix().compareTo(unit.getPrefix());
		}
		else {
			throw new ClassCastException("Cannot cast " + unit.getClass() +
					" into " + getClass());
		}
	}
	
	/**
	 * Returns true if the specified object is of type <i>Gram</i> and has
	 * the same prefix set.
	 * 
	 * @return true if and only if other is of type <i>Gram</i> and
	 * this.prefix.equals(other.prefix) returns true, false otherwise
	 */
	public boolean equals(Object other){
		if(getClass().equals(other.getClass())){
			AbstractMeasuringUnit otherUnit = (AbstractMeasuringUnit)other;
			return prefix.equals(otherUnit.prefix);
		}
		return false;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String getAbbreviation(){
		String preName = prefix.getSymbol();
		String unitName = ResourceBundle.getBundle(getLocaleBaseName())
		.getString("abbreviation");
		
		return preName + unitName;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String getAbbreviation(Locale locale) {
		String preName = prefix.getSymbol();
		String unitName =
		ResourceBundle.getBundle(getLocaleBaseName(), locale)
		.getString("abbreviation");
		
		return preName + unitName;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String getFullName() {
		return ResourceBundle.getBundle(getLocaleBaseName())
		.getString("fullname");
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String getFullName(Locale locale) {
		return ResourceBundle.getBundle(getLocaleBaseName(), locale)
		.getString("fullname");
	}
	
	/**
	 * {@inheritDoc}
	 */
	public Prefix getPrefix(){
		return prefix;
	}
	
	/**
	 * @return the hashcode for this object
	 */
	public int hashCode(){
		return getClass().hashCode() + 31 * prefix.hashCode();
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void setPrefix(Prefix prefix) {
		if(prefix == null){
			throw new IllegalArgumentException("prefix must not be null!");
		}
		
		this.prefix = prefix;
	}
	
	/**
	 * Fetches the base name for localization. Must be a fully qualified name
	 * conforming to the requirements of
	 * {@link ResourceBundle#getBundle(String, Locale, ClassLoader)} 
	 * 
	 * @return the fully qualified name of the base class for localization
	 */
	protected abstract String getLocaleBaseName();
}