package de.matthiasrothe.util.measuring;

import java.io.Serializable;
import java.util.Locale;

/**
 * A <i>Measurable</i> represents a number value and an associated measuring
 * unit.
 * <p>
 * Whereas the value can be an instance of any class extending
 * {@link Number}, the unit is an instance of a class implementing the
 * interface {@link MeasuringUnit}.
 *  
 * @author Matthias
 * 
 * @param <V> The type of the value, may be anything extending {@link Number}
 * @param <U> The type of the unit, may be anything extending {@link MeasuringUnit}
 */
public interface Measurable<V extends Number, U extends MeasuringUnit>
extends Serializable, Comparable<Measurable<V, U>> {
	
	/**
	 * <i>RetrievalMode</i> represents the two ways of retrieving a
	 * measure, either:<br>
	 * <p>
	 * a) With the abbreviated version of the unit, or<br>
	 * <p>
	 * b) With the full name version of the unit
	 * @author Matthias Rothe
	 */
	public enum RetrievalMode {
		abbreviated, fullname
	}
	
	/**
	 * Returns the unit used by this measurable.
	 * @return the unit
	 */
	public U getUnit();
	
	/**
	 * Returns the value of this measurable
	 * @return
	 */
	public V getValue();
	
	/**
	 * Returns the value and the unit according to the {@link RetrievalMode}
	 * given.
	 * 
	 * @return the value and unit
	 * @see MeasuringUnit#getAbbreviation()
	 * @see MeasuringUnit#getFullName()
	 */
	public String getValueAndUnit(RetrievalMode mode);
	
	/**
	 * Returns the value and the unit according to the {@link RetrievalMode}
	 * and {@link Locale} given.
	 * 
	 * @return the value and unit
	 * @see MeasuringUnit#getAbbreviation()
	 * @see MeasuringUnit#getFullName()
	 * @see Locale
	 */
	public String getValueAndUnit(RetrievalMode mode, Locale locale);
	
	/**
	 * Sets the value. Implementations should make sure that no null values are
	 * permitted.
	 * 
	 * @param value The new value
	 */
	public void setValue(V value);
}