package de.matthiasrothe.util.rating;

import java.io.*;

/**
 * 
 * @author Matthias Rothe
 *
 * @param <P>
 */
public class Rateable<P extends Comparable<? super P> & Serializable>
	implements Comparable<Rateable<P>>, Serializable {

	/**
	 * This enumeration lists the available sort orders for payload
	 * comparisons.
	 * 
	 * @author Matthias Rothe
	 * @since 2007-08-29
	 */
	public enum SortOrder {
		/**
		 * Ascending sort order
		 */
		ASC,
		
		/**
		 * Descending sort order
		 */
		DESC
	}
	
	private static final long serialVersionUID = 170201837620626062L;
	
	private Rating rating;
	private P payload;
	private boolean comparePayload = false;
	private SortOrder sortOrder = SortOrder.ASC;
	
	/**
	 *	Creates a blank <i>Rateable</i> object.
	 *	<p>
	 *	<b>Note:</b> This constructor is implemented to comply with the Java
	 *	Beans specification. Direct use is highly discouraged. If you use it
	 *	anyway, make sure you call all the necessary setters to set up the
	 *	object correctly.
	 */
	public Rateable(){
		//Standardkonstruktor fr Beankonformitt
	}
	
	/**
	 * Creates a <i>Rateable</i> with an initial rating and payload. The option
	 * of payload comparison will be disabled.
	 * <p>
	 * The parameters must not be null.
	 * 
	 * @param rating the rating of the payload
	 * @param payload the payload object
	 */
	public Rateable(Rating rating, P payload){
		if(rating == null || payload == null){
			throw new IllegalArgumentException("parameters must not be null!");
		}
		
		this.rating = rating;
		this.payload = payload;
	}
	
	/**
	 * Creates a <i>Rateable</i> with an initial rating and payload. The option
	 * of payload comparison will be enabled or disabled depending on the value
	 * of the comparePayload parameter. Depending on the sortOrder parameter
	 * the payload comparisons will result in ascending or descending ordering
	 * of the payload.
	 * <p>
	 * The parameters must not be null.
	 * 
	 * @param rating 			the rating of the payload
	 * @param payload 			the paylaod object
	 * @param comparePayload 	<code>true</code> enables / <code>false</code>
	 * 							disables the payload comparison option 
	 * @param sortOrder			the sort order used in payload comparisons
	 */
	public Rateable(Rating rating, P payload, boolean comparePayload,
					SortOrder sortOrder){
		
		this(rating, payload);
		
		if(sortOrder == null){
			throw new IllegalArgumentException("sortOrder must not be null!");
		}
		
		this.comparePayload = comparePayload;
		this.sortOrder = sortOrder;
	}
	
	/**
	 * Compares two <i>Rateable</i>s of the same payload type. If the option of
	 * payload comparison is disabled only the ratings will be compared. The
	 * result depends on the implementation of {@link Enum#compareTo(Enum)}.
	 * <p>
	 * If the option of payload comparison is enabled the ratings of the two
	 * objects will be compared first. If they are equal, have the same rating,
	 * the payload will be compared according to the set sort order. If no sort
	 * order is set the payload comparison will default to ascending sort
	 * order.
	 * 
	 * @param rateable the rateable this object is to be compared with
	 */
	public int compareTo(Rateable<P> rateable){
		int compare = rating.compareTo(rateable.rating);
		
		if(compare == 0 && comparePayload){
			compare = payload.compareTo(rateable.payload);
			
			if(sortOrder == SortOrder.DESC){
				compare *= -1;
			}
		}
		
		return compare;
	}
	
	/**
	 * Retrieves the rating.
	 * 
	 * @return the rating
	 */
	public Rating getRating(){
		return rating;
	}
	
	/**
	 * Retrieves the payload.
	 * 
	 * @return the payload object
	 */
	public P getPayload(){
		return payload;
	}
	
	/**
	 * Retrieves the sort order.
	 * 
	 * @return the sort order
	 */
	public SortOrder getSortOrder(){
		return sortOrder;
	}
	
	/**
	 * @return <code>true</code> if and only if the payload comparison option
	 * 			is enabled, false otherwise.
	 */
	public boolean isComparePayload(){
		return comparePayload;
	}
	
	/**
	 * Enables / disables the option of payload comparison in the case that
	 * both <i>Rateable</i>s have the same rating. The sort order can be set
	 * by calling {@link #setSortOrder(SortOrder)}. The default sort order is
	 * ascending.
	 * 
	 * @param comparePayload	<code>true</code> enables / <code>false</code>
	 * 							disables the payload comparison option
	 * 
	 * @see #setSortOrder(SortOrder)
	 */
	public void setComparePayload(boolean comparePayload) {
		this.comparePayload = comparePayload;
	}
	
	/**
	 * Sets the payload of this rateable. The payload object may be of any
	 * type that implements the {@link Comparable} and {@link Serializable}
	 * interfaces.
	 * 
	 * @param payload the payload to set
	 */
	public void setPayload(P payload) {
		this.payload = payload;
	}
	
	/**
	 * Sets the rating of this rateable.
	 * 
	 * @param rating the rating to set
	 */
	public void setRating(Rating rating) {
		this.rating = rating;
	}
	
	/**
	 * Sets the sort order in which payload objects are to be sorted.
	 * <p>
	 * Setting the sort order will only make sense if
	 * {@link #setComparePayload(boolean)} is also called with the parameter
	 * being <code>true</code>.
	 * 
	 * @param sortOrder the sortOrder to set
	 * @see #setComparePayload(boolean)
	 */
	public void setSortOrder(SortOrder sortOrder) {
		this.sortOrder = sortOrder;
	}
	
	/**
	 * For an instance of <i>Rateable</i> the toString() method acts a
	 * shorthand for {@link #getPayload()#toString()}. Therefore the actual
	 * String returned depends on the payload object.
	 * 
	 * @return the same as the call {@link #getPayload()#toString()} would
	 */
	public String toString(){
		return payload.toString();
	}
}